"""new field track_total.

Revision ID: 32e9fea590b7
Revises: eb8c7e20a080
Create Date: 2022-10-12 13:59:57.620933

"""
import sqlalchemy as sa

from alembic import op

# revision identifiers, used by Alembic.
revision = "32e9fea590b7"
down_revision = "eb8c7e20a080"
branch_labels = None
depends_on = None


def upgrade():
    with op.batch_alter_table("album", schema=None) as batch_op:
        batch_op.add_column(sa.Column("track_total", sa.Integer(), nullable=True))


def downgrade():
    with op.batch_alter_table("album", schema=None) as batch_op:
        batch_op.drop_column("track_total")



"""Adds the ``read`` command to moe."""

import argparse
import logging

import moe
import moe.cli
from moe.plugins import read, remove
from moe.util.cli import cli_query, query_parser

log = logging.getLogger("moe.cli.read")

__all__: list[str] = []


@moe.hookimpl
def add_command(cmd_parsers: argparse._SubParsersAction):
    """Adds the ``read`` command to Moe's CLI."""
    read_parser = cmd_parsers.add_parser(
        "read",
        description="Read item files and update moe with any changes.",
        help="read item files and update moe with any changes",
        parents=[query_parser],
    )
    read_parser.add_argument(
        "-r",
        "--remove",
        action="store_true",
        help="remove items that no longer exist on the filesystem",
    )
    read_parser.set_defaults(func=_parse_args)


def _parse_args(args: argparse.Namespace):
    """Parses the given commandline arguments.

    Tracks can be added as files or albums as directories.

    Args:
        args: Commandline arguments to parse.

    Raises:
        SystemExit: Path given does not exist.
    """
    items = cli_query(args.query, args.query_type)

    error_count = 0
    for item in items:
        try:
            read.read_item(item)
        except FileNotFoundError:
            if args.remove:
                remove.remove_item(item)
            else:
                log.error(f"Could not find item's path. [{item=!r}]")
                error_count += 1

    if error_count:
        raise SystemExit(1)



"""Reads item files and updates moe with any changes."""

import moe
from moe import config

from . import read_cli, read_core
from .read_core import *

__all__ = []
__all__.extend(read_core.__all__)


@moe.hookimpl
def plugin_registration():
    """Only register the cli sub-plugin if the cli is enabled."""
    config.CONFIG.pm.register(read_core, "read_core")
    if config.CONFIG.pm.has_plugin("cli"):
        config.CONFIG.pm.register(read_cli, "read_cli")



"""Core api for read items from the library."""

import logging

from moe.library import Album, LibItem, Track

__all__ = ["read_item"]

log = logging.getLogger("moe.read")


def read_item(item: LibItem):
    """Reads an item's file and updates the item with any changes.

    Args:
        item: Item to update.

    Raises:
        FileNotFoundError: Item's path doesn't exist.
    """
    log.debug(f"Reading item's file for changes. [{item=!r}]")

    if not item.path.exists():
        raise FileNotFoundError(f"Item's path does not exist. [path={item.path!r}]")

    if isinstance(item, Track):
        item.merge(Track.from_file(item.path), overwrite=True)
    elif isinstance(item, Album):
        item.merge(Album.from_dir(item.path), overwrite=True)

    log.info(f"Updated item from filesystem. [{item=!r}]")



"""new field barcode.

Revision ID: eb8c7e20a080
Revises: db3a470892c6
Create Date: 2022-10-11 14:51:44.788096

"""
import sqlalchemy as sa

from alembic import op

# revision identifiers, used by Alembic.
revision = "eb8c7e20a080"
down_revision = "db3a470892c6"
branch_labels = None
depends_on = None


def upgrade():
    with op.batch_alter_table("album", schema=None) as batch_op:
        batch_op.add_column(sa.Column("barcode", sa.String(), nullable=True))


def downgrade():
    with op.batch_alter_table("album", schema=None) as batch_op:
        batch_op.drop_column("barcode")
